#!/bin/bash

function create_app() {
    local orgs=$(ls /srv/orgs)
    cat app-parts/part01
    for org in $orgs ; do
	cat <<EOF
     - exec: sudo -u postgres createdb ${org}_discourse || exit 0
     - exec:
          stdin: |
            grant all privileges on database ${org}_discourse to discourse;
          cmd: sudo -u postgres psql ${org}_discourse
          raise_on_fail: false

     - exec: /bin/bash -c 'sudo -u postgres psql ${org}_discourse <<< "alter schema public owner to discourse;"'
     - exec: /bin/bash -c 'sudo -u postgres psql ${org}_discourse <<< "create extension if not exists hstore;"'
     - exec: /bin/bash -c 'sudo -u postgres psql ${org}_discourse <<< "create extension if not exists pg_trgm;"'

EOF
    done
    cat <<'EOF'

  after_code:
    - exec:
        cd: $home/plugins
        cmd:
          - mkdir -p plugins
#          - git clone https://github.com/discourse/docker_manager.git

    - exec:
        cd: $home
        cmd:
EOF
    cat <<EOF
          - git fetch --depth=1 origin tag $DISCOURSE_VERSION --no-tags
          - git checkout $DISCOURSE_VERSION

EOF
    cat <<'EOF'
  before_bundle_exec:
    - file:
        path: $home/config/multisite.yml
        contents: |
EOF
    
    for org in $orgs ; do
	local domain=$(cat /srv/orgs/${org}/domain)
	cat <<EOF
         ${org}:
           adapter: postgresql
           database: ${org}_discourse
           pool: 25
           timeout: 5000
           host_names:
             - $domain

EOF
    done
    cat <<EOF
  after_bundle_exec:
    - exec: cd /var/www/discourse && sudo -E -u discourse bundle exec rake multisite:migrate

## Any custom commands to run after building
run:
  - exec: echo "Beginning of custom commands"
  ## If you want to set the 'From' email address for your first registration, uncomment and change:
  ## After getting the first signup email, re-comment the line. It only needs to run once.
  #- exec: rails r "SiteSetting.notification_email='info@unconfigured.discourse.org'"
  - exec: echo "End of custom commands"
EOF
}

function backup() {
    cp /srv/discourse_docker/containers/app.yml /srv/discourse_docker/containers/app.yml-backup-$(date +%s)
}

function show_diff() {
    local backup=$(ls /srv/discourse_docker/containers/app.yml-backup-* | tail -1)
    diff -u $backup /srv/discourse_docker/containers/app.yml || false
}

backup
create_app > /srv/discourse_docker/containers/app.yml
show_diff
