#!/usr/bin/env python3

import os
import pickle
import sqlite3
import sys
from contextlib import ExitStack
from contextlib import contextmanager
from mailman.interfaces.member import DeliveryMode, DeliveryStatus, MemberRole


# A fake module to go with `Bouncer`.
class _Mailman:
    __path__ = 'src/mailman/commands/cli_import.py'


# A fake Mailman object with Bouncer class from Mailman 2.1, we don't use it
# but there are instances in the .pck files.
class _Bouncer:
    class _BounceInfo:
        pass


@contextmanager
def hacked_sys_modules(name, module):
    old_module = sys.modules.get(name)
    sys.modules[name] = module
    try:
        yield
    finally:
        if old_module is None:
            del sys.modules[name]
        else:
            sys.modules[name] = old_module


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d


conn = sqlite3.connect('mailman.db')
conn.row_factory = dict_factory
configs = {}

print("loading mailman.db")

c = conn.cursor()
c.execute('select * from mailman_mysql')
for row in c.fetchall():
    configs.setdefault(row['listname'], []).append(row)

print("number of members per list")

for n, rows in configs.items():
    print(f'{len(rows)} {n}')

print("repickling")

def replace(c, email, what, value):
    if c[what].get(email) == value:
        return False
    else:
        c[what][email] = value
        return True
    
with ExitStack() as resources:
    resources.enter_context(hacked_sys_modules('Mailman', _Mailman))
    resources.enter_context(
        hacked_sys_modules('Mailman.Bouncer', _Bouncer))

    for n, rows in configs.items():
        p = f'pickles/{n}-config.pck'
        if not os.path.exists(p):
            print(f'{p} not found, skipping')
            continue
        c = pickle.load(open(p, 'rb'), encoding='iso-8859-1')
        c['digest_members'] = {}
        modified = False
        addresses = set()
        for row in rows:
            addresses.add(row['address'])
            for row_name, config_name in (('lang', 'language'),
                                          ('name', 'usernames'),
                                          ('delivery_status', 'delivery_status'),
                                          ('user_options', 'user_options')):
                modified = replace(c, row['address'], config_name, row[row_name]) or modified
            modified = replace(c, row['address'], 'members', 0) or modified
        removed = set()
        config_a = set()
        for f, v in c.items():
            if not type(v) is dict:
                continue
            to_remove = set()
            for a, _ in v.items():
                if a not in addresses:
                    removed.add(a)
                    to_remove.add(a)
                else:
                    config_a.add(a)
            for a in to_remove:
                del v[a]
                modified = True
        config_only = config_a - addresses
        if modified:
            print(f'removed from config.pck {len(removed)} {removed}')
            print(f'pickle {n}')
            pickle.dump(c, open(p, 'wb'))
