#!/usr/bin/env python3

import os
import pickle
import sqlite3
import sys
import pprint
from contextlib import ExitStack
from contextlib import contextmanager
from mailman.interfaces.member import DeliveryMode, DeliveryStatus, MemberRole


# A fake module to go with `Bouncer`.
class _Mailman:
    __path__ = 'src/mailman/commands/cli_import.py'


# A fake Mailman object with Bouncer class from Mailman 2.1, we don't use it
# but there are instances in the .pck files.
class _Bouncer:
    class _BounceInfo:
        pass


@contextmanager
def hacked_sys_modules(name, module):
    old_module = sys.modules.get(name)
    sys.modules[name] = module
    try:
        yield
    finally:
        if old_module is None:
            del sys.modules[name]
        else:
            sys.modules[name] = old_module


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

conn = sqlite3.connect('mailman.db')
c = conn.cursor()
print([r[1] for r in c.execute("PRAGMA table_info('mailman_mysql')").fetchall()])
c.execute(f'select * from mailman_mysql where listname == "{sys.argv[1]}"')
for row in c.fetchall():
    print(row)

with ExitStack() as resources:
    resources.enter_context(hacked_sys_modules('Mailman', _Mailman))
    resources.enter_context(
        hacked_sys_modules('Mailman.Bouncer', _Bouncer))

    print("=============== pickle")
    p = f'pickles/{sys.argv[1]}-config.pck'
    c = pickle.load(open(p, 'rb'), encoding='iso-8859-1')
    pprint.pprint(c)
